package multithreading;

import model.GameBoard;

public abstract class GameBoardThreading extends GameBoard implements Runnable{
    private final Thread thread;
    private volatile boolean isRunning;
    private final String name;

    public GameBoardThreading(String name){
        thread = new Thread(this);
        isRunning = false;
        this.name = name;
    }
    
    public synchronized void start(){
        if (!isRunning){
            isRunning = true;
            thread.start();
        }
    }
    
    public synchronized void stop(){
        if (isRunning){
            thread.interrupt();
            while (isRunning){
                try {
                    wait();
                } catch (InterruptedException ex) {}
            }
        }
    }    

    @Override
    public void run() {
        logic();
        synchronized (GameBoardThreading.this){
            isRunning = false;
            notifyAll();
        }
        System.out.println(name + ": stopped");
    }
    
    protected abstract void logic();
}
